@@ -24,6 +24,8 @@ import org.locationtech.jts.geom.Geometry;
 import org.locationtech.jts.geom.GeometryFactory;
 import org.locationtech.jts.io.WKBReader;
 import org.locationtech.jts.io.ParseException;
+import org.locationtech.jts.io.gml2.GMLReader;
+import org.locationtech.jts.io.kml.KMLReader;
 
 public class Constructors {
 
@@ -118,14 +120,14 @@ public class Constructors {
     public static class ST_GeomFromWKT extends ScalarFunction {
         @DataTypeHint(value = "RAW", bridgedTo = org.locationtech.jts.geom.Geometry.class)
         public Geometry eval(@DataTypeHint("String") String wktString) throws ParseException {
-            return getGeometryByFileData(wktString, FileDataSplitter.WKT);
+            return org.apache.sedona.common.Constructors.geomFromWKT(wktString, 0);
         }
     }
 
     public static class ST_GeomFromText extends ScalarFunction {
         @DataTypeHint(value = "RAW", bridgedTo = org.locationtech.jts.geom.Geometry.class)
         public Geometry eval(@DataTypeHint("String") String wktString) throws ParseException {
-            return new ST_GeomFromWKT().eval(wktString);
+            return org.apache.sedona.common.Constructors.geomFromWKT(wktString, 0);
         }
     }
 
@@ -164,4 +166,23 @@ public class Constructors {
             return eval(value, null);
         }
     }
+
+    public static class ST_GeomFromGML extends ScalarFunction {
+        @DataTypeHint(value = "RAW", bridgedTo = org.locationtech.jts.geom.Geometry.class)
+        public Geometry eval(@DataTypeHint("String") String gml) throws ParseException {
+            GMLReader reader = new GMLReader();
+            try {
+                return reader.read(gml, new GeometryFactory());
+            } catch (Exception e) {
+                throw new ParseException(e);
+            }
+        }
+    }
+
+    public static class ST_GeomFromKML extends ScalarFunction {
+        @DataTypeHint(value = "RAW", bridgedTo = org.locationtech.jts.geom.Geometry.class)
+        public Geometry eval(@DataTypeHint("String") String kml) throws ParseException {
+            return new KMLReader().read(kml);
+        }
+    }
 }
