@@ -21,14 +21,17 @@ import org.junit.Test;
 import org.locationtech.jts.geom.Coordinate;
 import org.locationtech.jts.geom.Geometry;
 import org.locationtech.jts.geom.GeometryFactory;
+import org.locationtech.jts.geom.Polygon;
 import org.wololo.jts2geojson.GeoJSONReader;
 
 import java.util.ArrayList;
+import java.util.Arrays;
 import java.util.List;
 
 import static org.apache.flink.table.api.Expressions.$;
 import static org.apache.flink.table.api.Expressions.call;
 import static org.junit.Assert.assertEquals;
+import static org.junit.Assert.assertTrue;
 
 import org.apache.flink.api.common.typeinfo.TypeInformation;
 import org.apache.flink.api.common.typeinfo.BasicTypeInfo;
@@ -67,14 +70,14 @@ public class ConstructorTest extends TestBase{
 
         String expected = "POINT (1 2)";
 
-        assertEquals(result, expected);
+        assertEquals(expected, result);
     }
 
     @Test
     public void testPointFromText() {
         List<Row> data = createPointWKT(testDataSize);
         Row result = last(createPointTable(testDataSize));
-        assertEquals(result.toString(), data.get(data.size() - 1).toString());
+        assertEquals(data.get(data.size() - 1).getField(0).toString(), result.getField(0).toString());
     }
 
     @Test
@@ -86,7 +89,7 @@ public class ConstructorTest extends TestBase{
                         $(linestringColNames[1]));
         Row result = last(lineStringTable);
 
-        assertEquals(result.toString(), data.get(data.size() - 1).toString());
+        assertEquals(data.get(data.size() - 1).getField(0).toString(), result.getField(0).toString());
     }
 
     @Test
@@ -98,14 +101,14 @@ public class ConstructorTest extends TestBase{
                         $(linestringColNames[1]));
         Row result = last(lineStringTable);
 
-        assertEquals(result.toString(), data.get(data.size() - 1).toString());
+        assertEquals(data.get(data.size() - 1).getField(0).toString(), result.getField(0).toString());
     }
 
     @Test
     public void testPolygonFromText() {
         List<Row> data = createPolygonWKT(testDataSize);
         Row result = last(createPolygonTable(testDataSize));
-        assertEquals(result.toString(), data.get(data.size() - 1).toString());
+        assertEquals(data.get(data.size() - 1).getField(0).toString(), result.getField(0).toString());
     }
 
     @Test
@@ -116,7 +119,7 @@ public class ConstructorTest extends TestBase{
                 $(polygonColNames[0])).as(polygonColNames[0]),
                 $(polygonColNames[1]));
         Row result = last(geomTable);
-        assertEquals(result.toString(), data.get(data.size() - 1).toString());
+        assertEquals(data.get(data.size() - 1).getField(0).toString(), result.getField(0).toString());
     }
 
     @Test
@@ -127,7 +130,7 @@ public class ConstructorTest extends TestBase{
                         $(polygonColNames[0])).as(polygonColNames[0]),
                 $(polygonColNames[1]));
         Row result = last(geomTable);
-        assertEquals(result.toString(), data.get(data.size() - 1).toString());
+        assertEquals(data.get(data.size() - 1).getField(0).toString(), result.getField(0).toString());
     }
 
     @Test
@@ -144,10 +147,10 @@ public class ConstructorTest extends TestBase{
         coordinates[4] = coordinates[0];
         GeometryFactory geometryFactory = new GeometryFactory();
         Geometry geom = geometryFactory.createPolygon(coordinates);
-        assertEquals(geom.toString(), last(tableEnv.sqlQuery("SELECT ST_PolygonFromEnvelope(1, 100, 2, 200)"))
-                .getField(0).toString());
-        assertEquals(geom.toString(), last(tableEnv.sqlQuery("SELECT ST_PolygonFromEnvelope(1.0, 100.0, 2.0, 200.0)"))
-                .getField(0).toString());
+        assertEquals(last(tableEnv.sqlQuery("SELECT ST_PolygonFromEnvelope(1, 100, 2, 200)"))
+                .getField(0).toString(), geom.toString());
+        assertEquals(last(tableEnv.sqlQuery("SELECT ST_PolygonFromEnvelope(1.0, 100.0, 2.0, 200.0)"))
+                .getField(0).toString(), geom.toString());
 
     }
 
@@ -167,7 +170,7 @@ public class ConstructorTest extends TestBase{
                 .getFieldAs(0);
         String expectedGeom = reader.read(expectedGeoJSON).toText();
 
-        assertEquals(result, expectedGeom);
+        assertEquals(expectedGeom, result);
     }
 
     @Test
@@ -179,7 +182,7 @@ public class ConstructorTest extends TestBase{
         TypeInformation<?>[] colTypes = {
                 PrimitiveArrayTypeInfo.BYTE_PRIMITIVE_ARRAY_TYPE_INFO,
                 BasicTypeInfo.STRING_TYPE_INFO};
-        RowTypeInfo typeInfo = new RowTypeInfo(colTypes, polygonColNames);
+        RowTypeInfo typeInfo = new RowTypeInfo(colTypes, Arrays.copyOfRange(polygonColNames, 0, 2));
         DataStream<Row> wkbDS = env.fromCollection(data).returns(typeInfo);
         Table wkbTable = tableEnv.fromDataStream(wkbDS, $(polygonColNames[0]), $(polygonColNames[1]));
 
@@ -191,7 +194,7 @@ public class ConstructorTest extends TestBase{
 
         String expectedGeom = "LINESTRING (-2.1047439575195312 -0.354827880859375, -1.49606454372406 -0.6676061153411865)";
 
-        assertEquals(result, expectedGeom);
+        assertEquals(expectedGeom, result);
 
        }
 
@@ -199,7 +202,7 @@ public class ConstructorTest extends TestBase{
     public void testGeomFromGeoHash() {
         Integer precision = 2;
         List<Row> data = new ArrayList<>();
-        data.add(Row.of("2131s12fd", "polygon"));
+        data.add(Row.of("2131s12fd", "polygon", 0L));
 
         Table geohashTable = createTextTable(data, polygonColNames);
         Table geomTable = geohashTable
@@ -211,13 +214,13 @@ public class ConstructorTest extends TestBase{
                         .toString();
         String expectedGeom = "POLYGON ((-180 -39.375, -180 -33.75, -168.75 -33.75, -168.75 -39.375, -180 -39.375))";
 
-        assertEquals(result, expectedGeom);
+        assertEquals(expectedGeom, result);
     }
 
     @Test
     public void testGeomFromGeoHashNullPrecision() {
         List<Row> data = new ArrayList<>();
-        data.add(Row.of("2131s12fd", "polygon"));
+        data.add(Row.of("2131s12fd", "polygon", 0L));
 
         Table geohashTable = createTextTable(data, polygonColNames);
         Table geomTable = geohashTable
@@ -229,6 +232,48 @@ public class ConstructorTest extends TestBase{
                 .toString();
         String expectedGeom = "POLYGON ((-178.4168529510498 -37.69778251647949, -178.4168529510498 -37.697739601135254, -178.41681003570557 -37.697739601135254, -178.41681003570557 -37.69778251647949, -178.4168529510498 -37.69778251647949))";
 
-        assertEquals(result, expectedGeom);
+        assertEquals(expectedGeom, result);
+    }
+
+    @Test
+    public void testGeomFromGML() {
+        List<Row> data = new ArrayList<>();
+        String gml =
+                "<gml:Polygon>\n" +
+                "  <gml:outerBoundaryIs>\n" +
+                "    <gml:LinearRing>\n" +
+                "      <gml:coordinates>\n" +
+                "        0.0,0.0 0.0,1.0 1.0,1.0 1.0,0.0 0.0,0.0\n" +
+                "      </gml:coordinates>\n" +
+                "    </gml:LinearRing>\n" +
+                "  </gml:outerBoundaryIs>\n" +
+                "</gml:Polygon>";
+        data.add(Row.of(gml, "polygon", 0L));
+
+        Table wktTable = createTextTable(data, polygonColNames);
+        Table geomTable = wktTable.select(call(Constructors.ST_GeomFromGML.class.getSimpleName(),
+                $(polygonColNames[0])).as(polygonColNames[0]),
+                $(polygonColNames[1]));
+        assertTrue(first(geomTable).getField(0) instanceof Polygon);
+    }
+
+    @Test
+    public void testGeomFromKML() {
+        List<Row> data = new ArrayList<>();
+        String kml =
+                "<Polygon>\n" +
+                "  <outerBoundaryIs>\n" +
+                "    <LinearRing>\n" +
+                "      <coordinates>0.0,0.0 0.0,1.0 1.0,1.0 1.0,0.0 0.0,0.0</coordinates>\n" +
+                "    </LinearRing>\n" +
+                "  </outerBoundaryIs>\n" +
+                "</Polygon>";
+        data.add(Row.of(kml, "polygon", 0L));
+
+        Table wktTable = createTextTable(data, polygonColNames);
+        Table geomTable = wktTable.select(call(Constructors.ST_GeomFromKML.class.getSimpleName(),
+                $(polygonColNames[0])).as(polygonColNames[0]),
+                $(polygonColNames[1]));
+        assertTrue(first(geomTable).getField(0) instanceof Polygon);
     }
 }
