@@ -23,9 +23,9 @@ import org.apache.sedona.core.enums.{IndexType, JoinBuildSide}
 import org.apache.sedona.core.spatialOperator.JoinQuery.JoinParams
 
 object JoinParamsAdapter {
-  def createJoinParams(useIndex: Boolean = false, indexType: String, joinBuildSide: String): JoinParams = {
+  def createJoinParams(useIndex: Boolean = true, considerBoundaryIntersection: Boolean = false, indexType: String, joinBuildSide: String): JoinParams = {
     val buildSide = JoinBuildSide.getBuildSide(joinBuildSide)
     val currIndexType = IndexType.getIndexType(indexType)
-    new JoinParams(useIndex, currIndexType, buildSide)
+    new JoinParams(useIndex, considerBoundaryIntersection, currIndexType, buildSide)
   }
 }
\ No newline at end of file
