@@ -947,14 +947,11 @@ class TestPredicateJoin(TestBase):
             pointOnSurface = self.spark.sql("select ST_AsText(ST_PointOnSurface(ST_GeomFromText({})))".format(input_geom))
             assert pointOnSurface.take(1)[0][0] == expected_geom
 
-        '''
-        ST_AsEWKT Has not been implemented yet
-        tests2 = { "'LINESTRING(0 5 1, 0 0 1, 0 10 2)'":"POINT(0 0 1)" }
+        tests2 = { "'LINESTRING(0 5 1, 0 0 1, 0 10 2)'":"POINT Z(0 0 1)" }
 
         for input_geom, expected_geom in tests2.items():
-            pointOnSurface = self.spark.sql("select ST_AsEWKT(ST_PointOnSurface(ST_GeomFromEWKT({})))".format(input_geom))
+            pointOnSurface = self.spark.sql("select ST_AsEWKT(ST_PointOnSurface(ST_GeomFromWKT({})))".format(input_geom))
             assert pointOnSurface.take(1)[0][0] == expected_geom
-        '''
 
     def test_st_pointn(self):
         linestring = "'LINESTRING(0 0, 1 2, 2 4, 3 6)'"
@@ -1017,3 +1014,17 @@ class TestPredicateJoin(TestBase):
         for input_geom, expected_geom in tests.items():
             areal_geom = self.spark.sql("select ST_AsText(ST_BuildArea(ST_GeomFromText({})))".format(input_geom))
             assert areal_geom.take(1)[0][0] == expected_geom
+
+    def test_st_line_from_multi_point(self):
+        test_cases = {
+            "'POLYGON((-1 0 0, 1 0 0, 0 0 1, 0 1 0, -1 0 0))'": None,
+            "'LINESTRING(0 0, 1 2, 2 4, 3 6)'": None,
+            "'POINT(1 2)'": None,
+            "'MULTIPOINT((10 40), (40 30), (20 20), (30 10))'":
+                "LINESTRING (10 40, 40 30, 20 20, 30 10)",
+            "'MULTIPOINT((10 40 66), (40 30 77), (20 20 88), (30 10 99))'":
+                "LINESTRING Z(10 40 66, 40 30 77, 20 20 88, 30 10 99)"
+        }
+        for input_geom, expected_geom in test_cases.items():
+            line_geometry = self.spark.sql("select ST_AsText(ST_LineFromMultiPoint(ST_GeomFromText({})))".format(input_geom))
+            assert line_geometry.take(1)[0][0] == expected_geom
