@@ -19,6 +19,7 @@
 package org.apache.spark.sql.sedona_sql.strategy.join
 
 import org.apache.sedona.core.enums.IndexType
+import org.apache.sedona.core.spatialOperator.SpatialPredicate
 import org.apache.sedona.core.utils.SedonaConf
 import org.apache.spark.sql.{SparkSession, Strategy}
 import org.apache.spark.sql.catalyst.expressions.{And, Expression, LessThan, LessThanOrEqual}
@@ -28,15 +29,14 @@ import org.apache.spark.sql.execution.SparkPlan
 import org.apache.spark.sql.sedona_sql.expressions._
 
 
-
 case class JoinQueryDetection(
-  left: LogicalPlan,
-  right: LogicalPlan,
-  leftShape: Expression,
-  rightShape: Expression,
-  intersects: Boolean,
-  extraCondition: Option[Expression] = None,
-  radius: Option[Expression] = None
+                               left: LogicalPlan,
+                               right: LogicalPlan,
+                               leftShape: Expression,
+                               rightShape: Expression,
+                               spatialPredicate: SpatialPredicate,
+                               extraCondition: Option[Expression] = None,
+                               distance: Option[Expression] = None
 )
 
 /**
@@ -45,7 +45,7 @@ case class JoinQueryDetection(
   *
   * Plans `DistanceJoinExec` for inner joins on spatial relationship ST_Distance(a, b) < r.
   * 
-  * Plans `BroadcastIndexJoinExec for inner joins on spatial relationships with a broadcast hint.
+  * Plans `BroadcastIndexJoinExec` for inner joins on spatial relationships with a broadcast hint.
   */
 class JoinQueryDetector(sparkSession: SparkSession) extends Strategy {
 
@@ -56,19 +56,23 @@ class JoinQueryDetector(sparkSession: SparkSession) extends Strategy {
     extraCondition: Option[Expression] = None): Option[JoinQueryDetection] = {
       predicate match {
         case ST_Contains(Seq(leftShape, rightShape)) =>
-          Some(JoinQueryDetection(left, right, leftShape, rightShape, false, extraCondition))
+          Some(JoinQueryDetection(left, right, leftShape, rightShape, SpatialPredicate.CONTAINS, extraCondition))
         case ST_Intersects(Seq(leftShape, rightShape)) =>
-          Some(JoinQueryDetection(left, right, leftShape, rightShape, true, extraCondition))
+          Some(JoinQueryDetection(left, right, leftShape, rightShape, SpatialPredicate.INTERSECTS, extraCondition))
         case ST_Within(Seq(leftShape, rightShape)) =>
-          Some(JoinQueryDetection(right, left, rightShape, leftShape, false, extraCondition))
+          Some(JoinQueryDetection(left, right, leftShape, rightShape, SpatialPredicate.WITHIN, extraCondition))
+        case ST_Covers(Seq(leftShape, rightShape)) =>
+          Some(JoinQueryDetection(left, right, leftShape, rightShape, SpatialPredicate.COVERS, extraCondition))
+        case ST_CoveredBy(Seq(leftShape, rightShape)) =>
+          Some(JoinQueryDetection(left, right, leftShape, rightShape, SpatialPredicate.COVERED_BY, extraCondition))
         case ST_Overlaps(Seq(leftShape, rightShape)) =>
-          Some(JoinQueryDetection(right, left, rightShape, leftShape, false, extraCondition))
+          Some(JoinQueryDetection(left, right, leftShape, rightShape, SpatialPredicate.OVERLAPS, extraCondition))
         case ST_Touches(Seq(leftShape, rightShape)) =>
-          Some(JoinQueryDetection(left, right, leftShape, rightShape, true, extraCondition))
+          Some(JoinQueryDetection(left, right, leftShape, rightShape, SpatialPredicate.TOUCHES, extraCondition))
         case ST_Equals(Seq(leftShape, rightShape)) =>
-          Some(JoinQueryDetection(left, right, leftShape, rightShape, false, extraCondition))
+          Some(JoinQueryDetection(left, right, leftShape, rightShape, SpatialPredicate.EQUALS, extraCondition))
         case ST_Crosses(Seq(leftShape, rightShape)) =>
-          Some(JoinQueryDetection(right, left, rightShape, leftShape, false, extraCondition))
+          Some(JoinQueryDetection(left, right, leftShape, rightShape, SpatialPredicate.CROSSES, extraCondition))
         case _ => None
       }
     }
@@ -88,18 +92,20 @@ class JoinQueryDetector(sparkSession: SparkSession) extends Strategy {
           getJoinDetection(left, right, predicate, Some(extraCondition))
         case Some(And(extraCondition, predicate: ST_Predicate)) =>
           getJoinDetection(left, right, predicate, Some(extraCondition))
-        case Some(LessThanOrEqual(ST_Distance(Seq(leftShape, rightShape)), radius)) =>
-          Some(JoinQueryDetection(left, right, leftShape, rightShape, true, None, Some(radius)))
-        case Some(And(LessThanOrEqual(ST_Distance(Seq(leftShape, rightShape)), radius), extraCondition)) =>
-          Some(JoinQueryDetection(left, right, leftShape, rightShape, true, Some(extraCondition), Some(radius)))
-        case Some(And(extraCondition, LessThanOrEqual(ST_Distance(Seq(leftShape, rightShape)), radius))) =>
-          Some(JoinQueryDetection(left, right, leftShape, rightShape, true, Some(extraCondition), Some(radius)))
-        case Some(LessThan(ST_Distance(Seq(leftShape, rightShape)), radius)) =>
-          Some(JoinQueryDetection(left, right, leftShape, rightShape, false, None, Some(radius)))
-        case Some(And(LessThan(ST_Distance(Seq(leftShape, rightShape)), radius), extraCondition)) =>
-          Some(JoinQueryDetection(left, right, leftShape, rightShape, false, Some(extraCondition), Some(radius)))
-        case Some(And(extraCondition, LessThan(ST_Distance(Seq(leftShape, rightShape)), radius))) =>
-          Some(JoinQueryDetection(left, right, leftShape, rightShape, false, Some(extraCondition), Some(radius)))
+
+        // For distance joins we execute the actual predicate (condition) and not only extraConditions.
+        case Some(LessThanOrEqual(ST_Distance(Seq(leftShape, rightShape)), distance)) =>
+          Some(JoinQueryDetection(left, right, leftShape, rightShape, SpatialPredicate.INTERSECTS, condition, Some(distance)))
+        case Some(And(LessThanOrEqual(ST_Distance(Seq(leftShape, rightShape)), distance), _)) =>
+          Some(JoinQueryDetection(left, right, leftShape, rightShape, SpatialPredicate.INTERSECTS, condition, Some(distance)))
+        case Some(And(_, LessThanOrEqual(ST_Distance(Seq(leftShape, rightShape)), distance))) =>
+          Some(JoinQueryDetection(left, right, leftShape, rightShape, SpatialPredicate.INTERSECTS, condition, Some(distance)))
+        case Some(LessThan(ST_Distance(Seq(leftShape, rightShape)), distance)) =>
+          Some(JoinQueryDetection(left, right, leftShape, rightShape, SpatialPredicate.INTERSECTS, condition, Some(distance)))
+        case Some(And(LessThan(ST_Distance(Seq(leftShape, rightShape)), distance), _)) =>
+          Some(JoinQueryDetection(left, right, leftShape, rightShape, SpatialPredicate.INTERSECTS, condition, Some(distance)))
+        case Some(And(_, LessThan(ST_Distance(Seq(leftShape, rightShape)), distance))) =>
+          Some(JoinQueryDetection(left, right, leftShape, rightShape, SpatialPredicate.INTERSECTS, condition, Some(distance)))
         case _ =>
           None
       }
@@ -108,17 +114,17 @@ class JoinQueryDetector(sparkSession: SparkSession) extends Strategy {
 
       if ((broadcastLeft || broadcastRight) && sedonaConf.getUseIndex) {
         queryDetection match {
-          case Some(JoinQueryDetection(left, right, leftShape, rightShape, intersects, extraCondition, radius)) =>
-            planBroadcastJoin(left, right, Seq(leftShape, rightShape), intersects, sedonaConf.getIndexType, broadcastLeft, extraCondition, radius)
+          case Some(JoinQueryDetection(left, right, leftShape, rightShape, spatialPredicate, extraCondition, distance)) =>
+            planBroadcastJoin(left, right, Seq(leftShape, rightShape), spatialPredicate, sedonaConf.getIndexType, broadcastLeft, extraCondition, distance)
           case _ =>
             Nil
         }
       } else {
         queryDetection match {
-          case Some(JoinQueryDetection(left, right, leftShape, rightShape, intersects, extraCondition, None)) =>
-            planSpatialJoin(left, right, Seq(leftShape, rightShape), intersects, extraCondition)
-          case Some(JoinQueryDetection(left, right, leftShape, rightShape, intersects, extraCondition, Some(radius))) =>
-            planDistanceJoin(left, right, Seq(leftShape, rightShape), radius, intersects, extraCondition)
+          case Some(JoinQueryDetection(left, right, leftShape, rightShape, spatialPredicate, extraCondition, None)) =>
+            planSpatialJoin(left, right, Seq(leftShape, rightShape), spatialPredicate, extraCondition)
+          case Some(JoinQueryDetection(left, right, leftShape, rightShape, spatialPredicate, extraCondition, Some(distance))) =>
+            planDistanceJoin(left, right, Seq(leftShape, rightShape), distance, spatialPredicate, extraCondition)
           case None => 
             Nil
         }
@@ -133,8 +139,7 @@ class JoinQueryDetector(sparkSession: SparkSession) extends Strategy {
     * map to the output of the specified plan.
     */
   private def matches(expr: Expression, plan: LogicalPlan): Boolean =
-    expr.references.find(plan.outputSet.contains(_)).isDefined &&
-      expr.references.find(!plan.outputSet.contains(_)).isEmpty
+    expr.references.nonEmpty && expr.references.forall(plan.outputSet.contains(_))
 
   private def matchExpressionsToPlans(exprA: Expression,
                                       exprB: Expression,
@@ -151,17 +156,17 @@ class JoinQueryDetector(sparkSession: SparkSession) extends Strategy {
   private def planSpatialJoin(left: LogicalPlan,
                               right: LogicalPlan,
                               children: Seq[Expression],
-                              intersects: Boolean,
+                              spatialPredicate: SpatialPredicate,
                               extraCondition: Option[Expression] = None): Seq[SparkPlan] = {
     val a = children.head
     val b = children.tail.head
 
-    val relationship = if (intersects) "ST_Intersects" else "ST_Contains";
+    val relationship = s"ST_$spatialPredicate"
 
     matchExpressionsToPlans(a, b, left, right) match {
-      case Some((planA, planB, _)) =>
+      case Some((planA, planB, swappedLeftAndRight)) =>
         logInfo(s"Planning spatial join for $relationship relationship")
-        RangeJoinExec(planLater(planA), planLater(planB), a, b, intersects, extraCondition) :: Nil
+        RangeJoinExec(planLater(planA), planLater(planB), a, b, swappedLeftAndRight, spatialPredicate, extraCondition) :: Nil
       case None =>
         logInfo(
           s"Spatial join for $relationship with arguments not aligned " +
@@ -173,25 +178,23 @@ class JoinQueryDetector(sparkSession: SparkSession) extends Strategy {
   private def planDistanceJoin(left: LogicalPlan,
                                right: LogicalPlan,
                                children: Seq[Expression],
-                               radius: Expression,
-                               intersects: Boolean,
+                               distance: Expression,
+                               spatialPredicate: SpatialPredicate,
                                extraCondition: Option[Expression] = None): Seq[SparkPlan] = {
     val a = children.head
     val b = children.tail.head
 
-    val relationship = if (intersects) "ST_Distance <=" else "ST_Distance <";
-
     matchExpressionsToPlans(a, b, left, right) match {
-      case Some((planA, planB, _)) =>
-        if (radius.references.isEmpty || matches(radius, planA)) {
+      case Some((planA, planB, swappedLeftAndRight)) =>
+        if (distance.references.isEmpty || matches(distance, planA)) {
           logInfo("Planning spatial distance join")
-          DistanceJoinExec(planLater(planA), planLater(planB), a, b, radius, intersects, extraCondition) :: Nil
-        } else if (matches(radius, planB)) {
+          DistanceJoinExec(planLater(planA), planLater(planB), a, b, swappedLeftAndRight, distance, spatialPredicate, extraCondition) :: Nil
+        } else if (matches(distance, planB)) {
           logInfo("Planning spatial distance join")
-          DistanceJoinExec(planLater(planB), planLater(planA), b, a, radius, intersects, extraCondition) :: Nil
+          DistanceJoinExec(planLater(planB), planLater(planA), b, a, swappedLeftAndRight, distance, spatialPredicate, extraCondition) :: Nil
         } else {
           logInfo(
-            "Spatial distance join for ST_Distance with non-scalar radius " +
+            "Spatial distance join for ST_Distance with non-scalar distance " +
               "that is not a computation over just one side of the join is not supported")
           Nil
         }
@@ -206,19 +209,18 @@ class JoinQueryDetector(sparkSession: SparkSession) extends Strategy {
   private def planBroadcastJoin(left: LogicalPlan,
                                 right: LogicalPlan,
                                 children: Seq[Expression],
-                                intersects: Boolean,
+                                spatialPredicate: SpatialPredicate,
                                 indexType: IndexType,
                                 broadcastLeft: Boolean,
                                 extraCondition: Option[Expression],
-                                radius: Option[Expression]): Seq[SparkPlan] = {
+                                distance: Option[Expression]): Seq[SparkPlan] = {
     val a = children.head
     val b = children.tail.head
 
-    val relationship = radius match {
-      case Some(_) if intersects => "ST_Distance <="
-      case Some(_) if !intersects => "ST_Distance <"
-      case None if intersects => "ST_Intersects"
-      case None if !intersects => "ST_Contains"
+    val relationship = (distance, spatialPredicate) match {
+      case (Some(_), SpatialPredicate.INTERSECTS) => "ST_Distance <="
+      case (Some(_), _) => "ST_Distance <"
+      case (None, _) => s"ST_$spatialPredicate"
     }
 
     matchExpressionsToPlans(a, b, left, right) match {
@@ -227,15 +229,15 @@ class JoinQueryDetector(sparkSession: SparkSession) extends Strategy {
         val broadcastSide = if (broadcastLeft) LeftSide else RightSide
         val (leftPlan, rightPlan, streamShape, windowSide) = (broadcastSide, swapped) match {
           case (LeftSide, false) => // Broadcast the left side, windows on the left
-            (SpatialIndexExec(planLater(left), a, indexType, radius), planLater(right), b, LeftSide)
+            (SpatialIndexExec(planLater(left), a, indexType, distance), planLater(right), b, LeftSide)
           case (LeftSide, true) => // Broadcast the left side, objects on the left
             (SpatialIndexExec(planLater(left), b, indexType), planLater(right), a, RightSide)
           case (RightSide, false) => // Broadcast the right side, windows on the left
             (planLater(left), SpatialIndexExec(planLater(right), b, indexType), a, LeftSide)
           case (RightSide, true) => // Broadcast the right side, objects on the left
-            (planLater(left), SpatialIndexExec(planLater(right), a, indexType, radius), b, RightSide)
+            (planLater(left), SpatialIndexExec(planLater(right), a, indexType, distance), b, RightSide)
         }
-        BroadcastIndexJoinExec(leftPlan, rightPlan, streamShape, broadcastSide, windowSide, intersects, extraCondition, radius) :: Nil
+        BroadcastIndexJoinExec(leftPlan, rightPlan, streamShape, broadcastSide, windowSide, spatialPredicate, extraCondition, distance) :: Nil
       case None =>
         logInfo(
           s"Spatial join for $relationship with arguments not aligned " +
