@@ -22,15 +22,22 @@ package org.apache.sedona.sql
 import org.apache.commons.codec.binary.Hex
 import org.apache.sedona.sql.implicits._
 import org.apache.spark.sql.functions._
-import org.apache.spark.sql.sedona_sql.expressions.ST_MakeValid
 import org.apache.spark.sql.{DataFrame, Row}
-import org.geotools.geometry.jts.WKTReader2
 import org.locationtech.jts.algorithm.MinimumBoundingCircle
 import org.locationtech.jts.geom.{Geometry, Polygon}
 import org.locationtech.jts.io.WKTWriter
 import org.locationtech.jts.linearref.LengthIndexedLine
 import org.locationtech.jts.operation.distance3d.Distance3DOp
 import org.scalatest.{GivenWhenThen, Matchers}
+import org.xml.sax.InputSource
+import java.io.StringReader
+
+import javax.xml.parsers.DocumentBuilderFactory
+import javax.xml.xpath.XPathFactory
+import org.apache.sedona.sql.utils.GeometrySerializer
+import org.apache.spark.SparkException
+import org.geotools.referencing.CRS
+import org.opengis.referencing.{FactoryException, NoSuchAuthorityCodeException}
 
 class functionTestScala extends TestBaseScala with Matchers with GeometrySample with GivenWhenThen {
 
@@ -127,16 +134,67 @@ class functionTestScala extends TestBaseScala with Matchers with GeometrySample
       var polygonDf = sparkSession.sql("select ST_GeomFromWKT(polygontable._c0) as countyshape from polygontable")
       polygonDf.createOrReplaceTempView("polygondf")
       val polygon = "POLYGON ((110.54671 55.818002, 110.54671 55.143743, 110.940494 55.143743, 110.940494 55.818002, 110.54671 55.818002))"
-      val forceXYExpect = "POLYGON ((471596.69167460164 6185916.951191288, 471107.5623640998 6110880.974228167, 496207.109151055 6110788.804712435, 496271.31937046186 6185825.60569904, 471596.69167460164 6185916.951191288))"
+      // Floats don't have an exact decimal representation. String format varies across jvm:s. Do an approximate match.
+      val forceXYExpect = "POLYGON \\(\\(471596.69167460\\d* 6185916.95119\\d*, 471107.562364\\d* 6110880.97422\\d*, 496207.10915\\d* 6110788.80471\\d*, 496271.3193704\\d* 6185825.6056\\d*, 471596.6916746\\d* 6185916.95119\\d*\\)\\)"
 
       sparkSession.createDataset(Seq(polygon))
         .withColumn("geom", expr("ST_GeomFromWKT(value)"))
         .createOrReplaceTempView("df")
 
       val forceXYResult = sparkSession.sql(s"""select ST_Transform(ST_FlipCoordinates(ST_geomFromWKT('$polygon')),'EPSG:4326', 'EPSG:32649', false)""").rdd.map(row => row.getAs[Geometry](0).toString).collect()(0)
-      assert(forceXYResult == forceXYExpect)
+      forceXYResult should fullyMatch regex(forceXYExpect)
     }
 
+    it("Passed ST_transform WKT version"){
+      var polygonWktDf = sparkSession.read.format("csv").option("delimiter", "\t").option("header", "false").load(mixedWktGeometryInputLocation)
+      polygonWktDf.createOrReplaceTempView("polygontable")
+      var polygonDf = sparkSession.sql("select ST_GeomFromWKT(polygontable._c0) as countyshape from polygontable")
+      polygonDf.createOrReplaceTempView("polygondf")
+      val polygon = "POLYGON ((110.54671 55.818002, 110.54671 55.143743, 110.940494 55.143743, 110.940494 55.818002, 110.54671 55.818002))"
+      val forceXYExpect = "POLYGON \\(\\(471596.6916746\\d* 6185916.95119\\d*, 471107.562364\\d* 6110880.97422\\d*, 496207.10915\\d* 6110788.80471\\d*, 496271.319370\\d* 6185825.6056\\d*, 471596.6916746\\d* 6185916.95119\\d*\\)\\)"
+
+      val EPSG_TGT_CRS = CRS.decode("EPSG:32649")
+      val EPSG_TGT_WKT = EPSG_TGT_CRS.toWKT()
+      val EPSG_SRC_CRS = CRS.decode("EPSG:4326")
+      val EPSG_SRC_WKT = EPSG_SRC_CRS.toWKT()
+
+      sparkSession.createDataset(Seq(polygon))
+        .withColumn("geom", expr("ST_GeomFromWKT(value)"))
+        .createOrReplaceTempView("df")
+
+      val forceXYResult_TGT_WKT = sparkSession.sql(s"""select ST_Transform(ST_FlipCoordinates(ST_geomFromWKT('$polygon')),'EPSG:4326', '$EPSG_TGT_WKT', false)""").rdd.map(row => row.getAs[Geometry](0).toString).collect()(0)
+      forceXYResult_TGT_WKT should fullyMatch regex(forceXYExpect)
+
+      val forceXYResult_SRC_WKT = sparkSession.sql(s"""select ST_Transform(ST_FlipCoordinates(ST_geomFromWKT('$polygon')),'$EPSG_SRC_WKT', 'EPSG:32649', false)""").rdd.map(row => row.getAs[Geometry](0).toString).collect()(0)
+      forceXYResult_SRC_WKT should fullyMatch regex(forceXYExpect)
+
+      val forceXYResult_SRC_TGT_WKT = sparkSession.sql(s"""select ST_Transform(ST_FlipCoordinates(ST_geomFromWKT('$polygon')),'$EPSG_SRC_WKT', '$EPSG_TGT_WKT', false)""").rdd.map(row => row.getAs[Geometry](0).toString).collect()(0)
+      forceXYResult_SRC_TGT_WKT should fullyMatch regex(forceXYExpect)
+
+    }
+
+    it("Passed Function exception check"){
+      val EPSG_TGT_CRS = CRS.decode("EPSG:32649")
+      val EPSG_TGT_WKT = EPSG_TGT_CRS.toWKT()
+      val epsgFactoryErrorString = EPSG_TGT_WKT.substring(0,EPSG_TGT_WKT.length() - 1)
+      val epsgString = "EPSG:4326"
+      val epsgNoSuchAuthorityString = "EPSG:9377"
+      val polygon = "POLYGON ((110.54671 55.818002, 110.54671 55.143743, 110.940494 55.143743, 110.940494 55.818002, 110.54671 55.818002))"
+      import org.locationtech.jts.io.WKTReader
+      val reader = new WKTReader
+      val polygeom = reader.read(polygon)
+
+      intercept[FactoryException]{
+        val d = org.apache.sedona.common.Functions.transform(polygeom, epsgString, epsgFactoryErrorString)
+      }
+
+      intercept[NoSuchAuthorityCodeException]{
+        val d2 = org.apache.sedona.common.Functions.transform(polygeom, epsgString, epsgNoSuchAuthorityString)
+      }
+
+      }
+
+
     it("Passed ST_Intersection - intersects but not contains") {
 
       val testtable = sparkSession.sql("select ST_GeomFromWKT('POLYGON((1 1, 8 1, 8 8, 1 8, 1 1))') as a,ST_GeomFromWKT('POLYGON((2 2, 9 2, 9 9, 2 9, 2 2))') as b")
@@ -267,6 +325,8 @@ class functionTestScala extends TestBaseScala with Matchers with GeometrySample
       polygonDf.createOrReplaceTempView("polygondf")
       var wktDf = sparkSession.sql("select ST_AsText(countyshape) as wkt from polygondf")
       assert(polygonDf.take(1)(0).getAs[Geometry]("countyshape").toText.equals(wktDf.take(1)(0).getAs[String]("wkt")))
+      val wkt = sparkSession.sql("select ST_AsText(ST_SetSRID(ST_Point(1.0,1.0), 3021))").first().getString(0)
+      assert(wkt == "POINT (1 1)", "WKT should not contain SRID")
     }
 
     it("Passed ST_AsText 3D") {
@@ -313,6 +373,41 @@ class functionTestScala extends TestBaseScala with Matchers with GeometrySample
       assert(Hex.encodeHexString(df.first().get(0).asInstanceOf[Array[Byte]]) == s)
     }
 
+    it("Passed ST_AsBinary with srid") {
+      // ST_AsBinary should return a WKB.
+      // WKB does not contain any srid.
+      val df = sparkSession.sql("SELECT ST_AsBinary(ST_SetSRID(ST_Point(1.0,1.0), 3021)), ST_AsBinary(ST_Point(1.0,1.0))")
+      val withSrid: Array[Byte] = df.first().getAs(0)
+      val withoutSrid: Array[Byte] = df.first().getAs(1)
+      assert(withSrid.seq == withoutSrid.seq)
+    }
+
+    it("Passed ST_AsGML") {
+      val df = sparkSession.sql("SELECT ST_GeomFromWKT('POLYGON((1 1, 8 1, 8 8, 1 8, 1 1))') AS polygon")
+      df.createOrReplaceTempView("table")
+
+      val gmlDf = sparkSession.sql(
+        """select ST_AsGML(polygon) as geojson
+          |from table""".stripMargin)
+
+      val gml = DocumentBuilderFactory.newInstance.newDocumentBuilder.parse(new InputSource(new StringReader(gmlDf.first().getString(0))))
+      val coordinates = XPathFactory.newInstance.newXPath.evaluate("/Polygon/outerBoundaryIs/LinearRing/coordinates", gml)
+      assert(coordinates.trim === "1.0,1.0 8.0,1.0 8.0,8.0 1.0,8.0 1.0,1.0")
+    }
+
+    it("Passed ST_AsKML") {
+      val df = sparkSession.sql("SELECT ST_GeomFromWKT('POLYGON((1 1, 8 1, 8 8, 1 8, 1 1))') AS polygon")
+      df.createOrReplaceTempView("table")
+
+      val kmlDf = sparkSession.sql(
+        """select ST_AsKML(polygon) as geojson
+          |from table""".stripMargin)
+
+      val kml = DocumentBuilderFactory.newInstance.newDocumentBuilder.parse(new InputSource(new StringReader(kmlDf.first().getString(0))))
+      val coordinates = XPathFactory.newInstance.newXPath.evaluate("/Polygon/outerBoundaryIs/LinearRing/coordinates", kml)
+      assert(coordinates.trim === "1.0,1.0 8.0,1.0 8.0,8.0 1.0,8.0 1.0,1.0")
+    }
+
     it("Passed ST_SRID") {
       val df = sparkSession.sql("SELECT ST_SRID(ST_GeomFromWKT('POLYGON((1 1, 8 1, 8 8, 1 8, 1 1))'))")
       assert(df.first().getInt(0) == 0)
@@ -950,6 +1045,7 @@ class functionTestScala extends TestBaseScala with Matchers with GeometrySample
   }
 
   it("Should correctly remove using ST_RemovePoint") {
+    calculateStRemovePointOption("Linestring(0 0, 1 1, 1 0, 0 0)") shouldBe Some("LINESTRING (0 0, 1 1, 1 0)")
     calculateStRemovePointOption("Linestring(0 0, 1 1, 1 0, 0 0)", 0) shouldBe Some("LINESTRING (1 1, 1 0, 0 0)")
     calculateStRemovePointOption("Linestring(0 0, 1 1)", 0) shouldBe None
     calculateStRemovePointOption("Linestring(0 0, 1 1, 1 0, 0 0)", 1) shouldBe Some("LINESTRING (0 0, 1 0, 0 0)")
@@ -963,6 +1059,24 @@ class functionTestScala extends TestBaseScala with Matchers with GeometrySample
     calculateStRemovePointOption("MULTILINESTRING ((10 10, 20 20, 10 40, 10 10), (40 40, 30 30, 40 20, 30 10, 40 40))", 3) shouldBe None
   }
 
+  it("Should correctly set using ST_SetPoint") {
+    calculateStSetPointOption("Linestring(0 0, 1 1, 1 0, 0 0)", 0, "Point(0 1)") shouldBe Some("LINESTRING (0 1, 1 1, 1 0, 0 0)")
+    calculateStSetPointOption("Linestring(0 0, 1 1, 1 0, 0 0)", 1, "Point(0 1)") shouldBe Some("LINESTRING (0 0, 0 1, 1 0, 0 0)")
+    calculateStSetPointOption("Linestring(0 0, 1 1, 1 0, 0 0)", 2, "Point(0 1)") shouldBe Some("LINESTRING (0 0, 1 1, 0 1, 0 0)")
+    calculateStSetPointOption("Linestring(0 0, 1 1, 1 0, 0 0)", 3, "Point(0 1)") shouldBe Some("LINESTRING (0 0, 1 1, 1 0, 0 1)")
+    calculateStSetPointOption("Linestring(0 0, 1 1, 1 0, 0 0)", 4, "Point(0 1)") shouldBe None
+    calculateStSetPointOption("Linestring(0 0, 1 1, 1 0, 0 0)", -1, "Point(0 1)") shouldBe Some("LINESTRING (0 0, 1 1, 1 0, 0 1)")
+    calculateStSetPointOption("Linestring(0 0, 1 1, 1 0, 0 0)", -2, "Point(0 1)") shouldBe Some("LINESTRING (0 0, 1 1, 0 1, 0 0)")
+    calculateStSetPointOption("Linestring(0 0, 1 1, 1 0, 0 0)", -3, "Point(0 1)") shouldBe Some("LINESTRING (0 0, 0 1, 1 0, 0 0)")
+    calculateStSetPointOption("Linestring(0 0, 1 1, 1 0, 0 0)", -4, "Point(0 1)") shouldBe Some("LINESTRING (0 1, 1 1, 1 0, 0 0)")
+    calculateStSetPointOption("Linestring(0 0, 1 1, 1 0, 0 0)", -5, "Point(0 1)") shouldBe None
+    calculateStSetPointOption("POINT(0 1)", 0, "Point(0 1)") shouldBe None
+    calculateStSetPointOption("POLYGON ((0 0, 0 5, 5 5, 5 0, 0 0), (1 1, 2 1, 2 2, 1 2, 1 1))", 0, "Point(0 1)") shouldBe None
+    calculateStSetPointOption("GEOMETRYCOLLECTION (POINT (40 10), LINESTRING (10 10, 20 20, 10 40))", 0, "Point(0 1)") shouldBe None
+    calculateStSetPointOption("MULTIPOLYGON (((30 20, 45 40, 10 40, 30 20)), ((15 5, 40 10, 10 20, 5 10, 15 5)))", 0, "Point(0 1)") shouldBe None
+    calculateStSetPointOption("MULTILINESTRING ((10 10, 20 20, 10 40, 10 10), (40 40, 30 30, 40 20, 30 10, 40 40))", 0, "Point(0 1)") shouldBe None
+  }
+
   it("Should pass ST_IsRing") {
     calculateStIsRing("LINESTRING(0 0, 0 1, 1 0, 1 1, 0 0)") shouldBe Some(false)
     calculateStIsRing("LINESTRING(2 0, 2 2, 3 3)") shouldBe Some(false)
@@ -1106,6 +1220,14 @@ class functionTestScala extends TestBaseScala with Matchers with GeometrySample
   private def wktToDf(wkt: String): DataFrame =
     Seq(Tuple1(wktReader.read(wkt))).toDF("geom")
 
+  private def calculateStRemovePointOption(wkt: String): Option[String] =
+    calculateStRemovePoint(wkt).headOption
+
+  private def calculateStRemovePoint(wkt: String): Array[String] =
+    wktToDf(wkt).selectExpr(s"ST_RemovePoint(geom) as geom")
+      .filter("geom is not null")
+      .selectExpr("ST_AsText(geom)").as[String].collect()
+
   private def calculateStRemovePointOption(wkt: String, index: Int): Option[String] =
     calculateStRemovePoint(wkt, index).headOption
 
@@ -1114,6 +1236,15 @@ class functionTestScala extends TestBaseScala with Matchers with GeometrySample
       .filter("geom is not null")
       .selectExpr("ST_AsText(geom)").as[String].collect()
 
+  private def calculateStSetPointOption(wktA: String, index: Int, wktB: String): Option[String] =
+    calculateStSetPoint(wktA, index, wktB).headOption
+
+  private def calculateStSetPoint(wktA: String, index: Int, wktB: String): Array[String] =
+    Seq(Tuple3(wktReader.read(wktA), index, wktReader.read(wktB))).toDF("geomA", "index", "geomB")
+      .selectExpr(s"ST_SetPoint(geomA, index, geomB) as geom")
+      .filter("geom is not null")
+      .selectExpr("ST_AsText(geom)").as[String].collect()
+
   it("Passed ST_NumGeometries") {
     Given("Some different types of geometries in a DF")
     // Test data
@@ -1291,25 +1422,22 @@ class functionTestScala extends TestBaseScala with Matchers with GeometrySample
         -> "POINT Z(0 0 1)"
     )
 
-        for((inputGeom, expectedGeom) <- geomTestCases1) {
-      var df = sparkSession.sql(s"select ST_AsText(ST_PointOnSurface(ST_GeomFromText($inputGeom)))")
-      var result = df.collect()
+    for((inputGeom, expectedGeom) <- geomTestCases1) {
+      val df = sparkSession.sql(s"select ST_AsText(ST_PointOnSurface(ST_GeomFromText($inputGeom)))")
+      val result = df.collect()
       assert(result.head.get(0).asInstanceOf[String]==expectedGeom)
     }
 
-    /* ST_AsEWKT Has not been implemented yet
-
     val geomTestCases2 = Map(
       "'LINESTRING(0 5 1, 0 0 1, 0 10 2)'"
-        -> "POINT (0 0 1)"
+        -> "POINT Z(0 0 1)"
     )
 
     for((inputGeom, expectedGeom) <- geomTestCases2) {
-      var df = sparkSession.sql(s"select ST_AsEWKT(ST_PointOnSurface(ST_GeomFromEWKT($inputGeom)))")
-      var result = df.collect()
+      val df = sparkSession.sql(s"select ST_AsEWKT(ST_PointOnSurface(ST_GeomFromWKT($inputGeom)))")
+      val result = df.collect()
       assert(result.head.get(0).asInstanceOf[String]==expectedGeom)
     }
-    */
   }
 
   it ("Should pass ST_Reverse") {
@@ -1575,6 +1703,10 @@ class functionTestScala extends TestBaseScala with Matchers with GeometrySample
     assert(functionDf.first().get(0) == null)
     functionDf = sparkSession.sql("select ST_BuildArea(null)")
     assert(functionDf.first().get(0) == null)
+    functionDf = sparkSession.sql("select ST_Normalize(null)")
+    assert(functionDf.first().get(0) == null)
+    functionDf = sparkSession.sql("select ST_LineFromMultiPoint(null)")
+    assert(functionDf.first().get(0) == null)
   }
 
   it ("Should pass St_CollectionExtract") {
@@ -1589,4 +1721,29 @@ class functionTestScala extends TestBaseScala with Matchers with GeometrySample
     assert(df.selectExpr("ST_AsText(ST_CollectionExtract(geom, 2))").collect().head.get(0) == "MULTILINESTRING EMPTY")
     assert(df.selectExpr("ST_AsText(ST_CollectionExtract(geom))").collect().head.get(0) == "MULTIPOINT ((40 10), (40 10))")
   }
-}
\ No newline at end of file
+
+  it("Should pass ST_Normalize") {
+    val df = sparkSession.sql("SELECT ST_AsEWKT(ST_Normalize(ST_GeomFromWKT('POLYGON((0 1, 1 1, 1 0, 0 0, 0 1))')))")
+    assert(df.first().get(0).asInstanceOf[String] == "POLYGON ((0 0, 0 1, 1 1, 1 0, 0 0))")
+  }
+
+  it ("Should pass ST_LineFromMultiPoint") {
+    val geomTestCases = Map(
+      "'POLYGON((-1 0 0, 1 0 0, 0 0 1, 0 1 0, -1 0 0))'"
+        -> null,
+      "'LINESTRING(0 0, 1 2, 2 4, 3 6)'"
+        -> null,
+      "'POINT(1 2)'"
+        -> null,
+      "'MULTIPOINT((10 40), (40 30), (20 20), (30 10))'"
+        -> "LINESTRING (10 40, 40 30, 20 20, 30 10)",
+      "'MULTIPOINT((10 40 66), (40 30 77), (20 20 88), (30 10 99))'"
+        -> "LINESTRING Z(10 40 66, 40 30 77, 20 20 88, 30 10 99)"
+    )
+    for((inputGeom, expectedGeom) <- geomTestCases) {
+      var df = sparkSession.sql(s"select ST_AsText(ST_LineFromMultiPoint(ST_GeomFromText($inputGeom)))")
+      var result = df.collect()
+      assert(result.head.get(0).asInstanceOf[String]==expectedGeom)
+    }
+  }
+}
