@@ -21,6 +21,7 @@ package org.apache.sedona.sql
 
 import org.apache.sedona.core.utils.SedonaConf
 import org.apache.spark.sql.Row
+import org.apache.spark.sql.execution.ExplainMode
 import org.apache.spark.sql.types._
 import org.locationtech.jts.geom.Geometry
 import org.locationtech.jts.io.WKTWriter
@@ -76,7 +77,7 @@ class predicateJoinTestScala extends TestBaseScala {
 
       var rangeJoinDf = sparkSession.sql("select * from polygondf, pointdf where ST_Touches(polygondf.polygonshape,pointdf.pointshape) ")
 
-      assert(rangeJoinDf.count() == 1000)
+      assert(rangeJoinDf.count() == 0)
     }
 
     it("Passed ST_Within in a join") {
@@ -108,7 +109,7 @@ class predicateJoinTestScala extends TestBaseScala {
 
       var rangeJoinDf = sparkSession.sql("select * from polygondf, polygonodf where ST_Overlaps(polygondf.polygonshape, polygonodf.polygonshape)")
 
-      assert(rangeJoinDf.count() == 57)
+      assert(rangeJoinDf.count() == 15)
     }
 
     it("Passed ST_Crosses in a join") {
@@ -124,6 +125,41 @@ class predicateJoinTestScala extends TestBaseScala {
 
       var rangeJoinDf = sparkSession.sql("select * from polygondf, pointdf where ST_Crosses(pointdf.pointshape, polygondf.polygonshape) ")
 
+      assert(rangeJoinDf.count() == 0)
+    }
+
+    it("Passed ST_Covers in a join") {
+      val sedonaConf = new SedonaConf(sparkSession.sparkContext.getConf)
+      println(sedonaConf)
+
+      var polygonCsvDf = sparkSession.read.format("csv").option("delimiter", ",").option("header", "false").load(csvPolygonInputLocation)
+      polygonCsvDf.createOrReplaceTempView("polygontable")
+      var polygonDf = sparkSession.sql("select ST_PolygonFromEnvelope(cast(polygontable._c0 as Decimal(24,20)),cast(polygontable._c1 as Decimal(24,20)), cast(polygontable._c2 as Decimal(24,20)), cast(polygontable._c3 as Decimal(24,20))) as polygonshape from polygontable")
+      polygonDf.createOrReplaceTempView("polygondf")
+
+      var pointCsvDF = sparkSession.read.format("csv").option("delimiter", ",").option("header", "false").load(csvPointInputLocation)
+      pointCsvDF.createOrReplaceTempView("pointtable")
+      var pointDf = sparkSession.sql("select ST_Point(cast(pointtable._c0 as Decimal(24,20)),cast(pointtable._c1 as Decimal(24,20))) as pointshape from pointtable")
+      pointDf.createOrReplaceTempView("pointdf")
+
+      var rangeJoinDf = sparkSession.sql("select * from polygondf, pointdf where ST_Covers(polygondf.polygonshape,pointdf.pointshape) ")
+
+      assert(rangeJoinDf.count() == 1000)
+    }
+
+    it("Passed ST_CoveredBy in a join") {
+      var polygonCsvDf = sparkSession.read.format("csv").option("delimiter", ",").option("header", "false").load(csvPolygonInputLocation)
+      polygonCsvDf.createOrReplaceTempView("polygontable")
+      var polygonDf = sparkSession.sql("select ST_PolygonFromEnvelope(cast(polygontable._c0 as Decimal(24,20)),cast(polygontable._c1 as Decimal(24,20)), cast(polygontable._c2 as Decimal(24,20)), cast(polygontable._c3 as Decimal(24,20))) as polygonshape from polygontable")
+      polygonDf.createOrReplaceTempView("polygondf")
+
+      var pointCsvDF = sparkSession.read.format("csv").option("delimiter", ",").option("header", "false").load(csvPointInputLocation)
+      pointCsvDF.createOrReplaceTempView("pointtable")
+      var pointDf = sparkSession.sql("select ST_Point(cast(pointtable._c0 as Decimal(24,20)),cast(pointtable._c1 as Decimal(24,20))) as pointshape from pointtable")
+      pointDf.createOrReplaceTempView("pointdf")
+
+      var rangeJoinDf = sparkSession.sql("select * from polygondf, pointdf where ST_CoveredBy(pointdf.pointshape, polygondf.polygonshape) ")
+
       assert(rangeJoinDf.count() == 1000)
     }
 
@@ -183,7 +219,7 @@ class predicateJoinTestScala extends TestBaseScala {
       assert(distanceJoinDf.count() == 2998)
     }
 
-    it("Passed ST_Distance < radius in a join") {
+    it("Passed ST_Distance < distance in a join") {
       var pointCsvDF1 = sparkSession.read.format("csv").option("delimiter", ",").option("header", "false").load(csvPointInputLocation)
       pointCsvDF1.createOrReplaceTempView("pointtable")
       var pointDf1 = sparkSession.sql("select ST_Point(cast(pointtable._c0 as Decimal(24,20)),cast(pointtable._c1 as Decimal(24,20))) as pointshape1 from pointtable")
@@ -199,6 +235,23 @@ class predicateJoinTestScala extends TestBaseScala {
       assert(distanceJoinDf.count() == 2998)
     }
 
+    it("Passed ST_Distance < distance with LineString in a join") {
+      assert(sparkSession.sql(
+        """
+          |select *
+          |from (select ST_LineFromText('LineString(1 1, 1 3, 3 3)') as geom) a
+          |join (select ST_Point(2.0,2.0) as geom) b
+          |on ST_Distance(a.geom, b.geom) < 0.1
+          |""".stripMargin).isEmpty)
+      assert(sparkSession.sql(
+        """
+          |select *
+          |from (select ST_LineFromText('LineString(1 1, 1 4)') as geom) a
+          |join (select ST_Point(1.0,5.0) as geom) b
+          |on ST_Distance(a.geom, b.geom) < 1.5
+          |""".stripMargin).count() == 1)
+    }
+
     it("Passed ST_Contains in a range and join") {
       var polygonCsvDf = sparkSession.read.format("csv").option("delimiter", ",").option("header", "false").load(csvPolygonInputLocation)
       polygonCsvDf.createOrReplaceTempView("polygontable")
