@@ -29,47 +29,41 @@ class standardVizOperatorTest extends TestBaseScala {
     it("Generate a single image") {
       spark.sql(
         """
-          |CREATE OR REPLACE TEMP VIEW pixels AS
           |SELECT pixel, shape FROM pointtable
           |LATERAL VIEW EXPLODE(ST_Pixelize(shape, 256, 256, ST_PolygonFromEnvelope(-126.790180,24.863836,-64.630926,50.000)) ) AS pixel
-        """.stripMargin)
+        """.stripMargin).createOrReplaceTempView("pixels")
       spark.sql(
         """
-          |CREATE OR REPLACE TEMP VIEW pixelaggregates AS
           |SELECT pixel, count(*) as weight
           |FROM pixels
           |GROUP BY pixel
-        """.stripMargin)
+        """.stripMargin).createOrReplaceTempView("pixelaggregates")
       spark.sql(
         """
-          |CREATE OR REPLACE TEMP VIEW images AS
           |SELECT ST_Render(pixel, ST_Colorize(weight, (SELECT max(weight) FROM pixelaggregates), 'red')) AS image
           |FROM pixelaggregates
-        """.stripMargin)
+        """.stripMargin).createOrReplaceTempView("images")
       var image = spark.table("images").take(1)(0)(0).asInstanceOf[ImageSerializableWrapper].getImage
       var imageGenerator = new ImageGenerator
       imageGenerator.SaveRasterImageAsLocalFile(image, System.getProperty("user.dir")+"/target/points", ImageType.PNG)
-      spark.sql(
+      val imageString = spark.sql(
         """
-          |CREATE OR REPLACE TEMP VIEW imagestring AS
           |SELECT ST_EncodeImage(image)
           |FROM images
         """.stripMargin)
-      spark.table("imagestring").show(1)
+      imageString.show(1)
     }
 
     it("Generate a single image using a fat query") {
       spark.sql(
         """
-          |CREATE OR REPLACE TEMP VIEW boundtable AS
           |SELECT ST_Envelope_Aggr(shape) as bound FROM pointtable
-        """.stripMargin)
+        """.stripMargin).createOrReplaceTempView("boundtable")
       spark.sql(
         """
-          |CREATE OR REPLACE TEMP VIEW pixels AS
           |SELECT pixel, shape FROM pointtable
           |LATERAL VIEW EXPLODE(ST_Pixelize(ST_Transform(ST_FlipCoordinates(shape), 'epsg:4326','epsg:3857'), 256, 256, (SELECT ST_Transform(ST_FlipCoordinates(bound), 'epsg:4326','epsg:3857') FROM boundtable))) AS pixel
-        """.stripMargin)
+        """.stripMargin).createOrReplaceTempView("pixels")
       spark.sql(
         """
           |CREATE OR REPLACE TEMP VIEW pixelaggregates AS
@@ -77,22 +71,20 @@ class standardVizOperatorTest extends TestBaseScala {
           |FROM pixels
           |GROUP BY pixel
         """.stripMargin)
-      spark.sql(
+      val images = spark.sql(
         """
-          |CREATE OR REPLACE TEMP VIEW images AS
           |SELECT ST_EncodeImage(ST_Render(pixel, ST_Colorize(weight, (SELECT max(weight) FROM pixelaggregates)))) AS image, (SELECT ST_AsText(bound) FROM boundtable) AS boundary
           |FROM pixelaggregates
         """.stripMargin)
-      spark.table("images").show(1)
+      images.show(1)
     }
 
     it("Passed the pipeline on points") {
       spark.sql(
         """
-          |CREATE OR REPLACE TEMP VIEW pixels AS
           |SELECT pixel, shape FROM pointtable
           |LATERAL VIEW EXPLODE(ST_Pixelize(shape, 1000, 800, ST_PolygonFromEnvelope(-126.790180,24.863836,-64.630926,50.000))) AS pixel
-        """.stripMargin)
+        """.stripMargin).createOrReplaceTempView("pixels")
       spark.sql(
         """
           |CREATE OR REPLACE TEMP VIEW pixelaggregates AS
@@ -107,10 +99,9 @@ class standardVizOperatorTest extends TestBaseScala {
     it("Passed the pipeline on polygons") {
       spark.sql(
         """
-          |CREATE OR REPLACE TEMP VIEW pixels AS
           |SELECT pixel, rate, shape FROM usdata
           |LATERAL VIEW EXPLODE(ST_Pixelize(shape, 1000, 1000, ST_PolygonFromEnvelope(-126.790180,24.863836,-64.630926,50.000))) AS pixel
-        """.stripMargin)
+        """.stripMargin).createOrReplaceTempView("pixels")
       spark.sql(
         """
           |CREATE OR REPLACE TEMP VIEW pixelaggregates AS
@@ -118,17 +109,11 @@ class standardVizOperatorTest extends TestBaseScala {
           |FROM pixels
           |GROUP BY pixel
         """.stripMargin)
-      spark.sql(
+      val imageDf = spark.sql(
         """
-          |CREATE OR REPLACE TEMP VIEW images AS
           |SELECT ST_Render(pixel, ST_Colorize(weight, (SELECT max(weight) FROM pixelaggregates))) AS image
           |FROM pixelaggregates
         """.stripMargin)
-      var imageDf = spark.sql(
-        """
-          |SELECT image
-          |FROM images
-        """.stripMargin)
       var image = imageDf.take(1)(0)(0).asInstanceOf[ImageSerializableWrapper].getImage
       var imageGenerator = new ImageGenerator
       imageGenerator.SaveRasterImageAsLocalFile(image, System.getProperty("user.dir")+"/target/polygons", ImageType.PNG)
@@ -138,10 +123,9 @@ class standardVizOperatorTest extends TestBaseScala {
       var zoomLevel = 2
       spark.sql(
         """
-          |CREATE OR REPLACE TEMP VIEW pixels AS
           |SELECT pixel, shape FROM pointtable
           |LATERAL VIEW EXPLODE(ST_Pixelize(shape, 1000, 1000, ST_PolygonFromEnvelope(-126.790180,24.863836,-64.630926,50.000))) AS pixel
-        """.stripMargin)
+        """.stripMargin).createOrReplaceTempView("pixels")
       spark.sql(
         """
           |CREATE OR REPLACE TEMP VIEW pixelaggregates AS
@@ -151,18 +135,16 @@ class standardVizOperatorTest extends TestBaseScala {
         """.stripMargin)
       spark.sql(
         s"""
-          |CREATE OR REPLACE TEMP VIEW pixel_weights AS
           |SELECT pixel, weight, ST_TileName(pixel, $zoomLevel) AS pid
           |FROM pixelaggregates
-        """.stripMargin)
-      spark.sql(
+        """.stripMargin).createOrReplaceTempView("pixel_weights")
+      val images = spark.sql(
         s"""
-          |CREATE OR REPLACE TEMP VIEW images AS
           |SELECT ST_Render(pixel, ST_Colorize(weight, (SELECT max(weight) FROM pixelaggregates)), $zoomLevel) AS image
           |FROM pixel_weights
           |GROUP BY pid
-        """.stripMargin).explain()
-      spark.table("images").show(1)
+        """.stripMargin)
+      images.show(1)
     }
   }
 }
